@@ -20,7 +20,7 @@ limit.data <- function(df, n) {
 #' @param folder The folder containing the .info files
 #' @return The paths to all found .info-files
 #' @export
-#' @examples 
+#' @examples
 #' path <- system.file("extdata", "ONE_PLUS_LAMDA_EA", package="IOHanalyzer")
 #' scan_index_file(path)
 scan_index_file <- function(folder) {
@@ -33,14 +33,14 @@ scan_index_file <- function(folder) {
 #' @param fname The path to the .info file
 #' @return The data contained in the .info file
 #' @export
-#' @examples 
+#' @examples
 #' path <- system.file("extdata", "ONE_PLUS_LAMDA_EA", package="IOHanalyzer")
 #' info <- read_index_file(file.path(path,"IOHprofiler_f1_i1.info"))
 read_index_file <- function(fname) {
   tryCatch(
     read_index_file__IOH(fname),
-    warning = function(e) read_index_file__BIOBJ_COCO(fname),
-    error = function(e) read_index_file__BIOBJ_COCO(fname),
+    warning = function(e) read_index_file__COCO(fname),
+    error = function(e) read_index_file__COCO(fname),
     finally = function(e) stop(paste0('Error in reading .info files ', e))
   )
 }
@@ -81,9 +81,13 @@ read_index_file__IOH <- function(fname) {
         for (name in .[1, ]) {
           value <- ans[[name]]
           ans[[name]] <- gsub("'", '', value)
-          value <- suppressWarnings(as.numeric(value)) # convert quoted numeric values to numeric
-          if (!is.na(value))
-            ans[[name]] <- value
+          
+          if (name == 'maximization')
+            value <- as.logical(value)
+          else
+            value <- suppressWarnings(as.numeric(value)) # convert quoted numeric values to numeric
+          
+          if (!is.na(value)) ans[[name]] <- value
         }
         ans
       }
@@ -120,6 +124,80 @@ read_index_file__IOH <- function(fname) {
   data
 }
 
+#' Read single-objective COCO-based .info files and extract information
+#'
+#' @param fname The path to the .info file
+#' @return The data contained in the .info file
+#' @noRd
+read_index_file__COCO <- function(fname) {
+  f <- file(fname, 'r')
+  path <- dirname(fname)
+  data <- list()
+  i <- 1
+  while (TRUE) {
+    
+    lines <- suppressWarnings(readLines(f, n = 3))  # read header and comments
+    if (length(lines) < 3) {
+      break
+    }
+    comment <- lines[2]
+    name_value <- as.vector(unlist(as.list(read.csv(text = lines[1], header = F, quote = "'"))))
+    
+    header <- trimws(name_value) %>% {
+      regmatches(., regexpr("=", .), invert = T)  # match the first appearance of '='
+    } %>%
+      unlist %>%
+      trimws %>%
+      matrix(nrow = 2) %>% {
+        ans <- as.list(.[2, ])
+        names(ans) <- .[1, ]
+        for (name in .[1, ]) {
+          value <- ans[[name]]
+          ans[[name]] <- gsub("'", '', value)
+          value <- suppressWarnings(as.numeric(value)) # convert quoted numeric values to numeric
+          if (!is.na(value))
+            ans[[name]] <- value
+        }
+        ans
+      }
+    
+    names(header) <- gsub('algorithm', 'algId', names(header))
+    
+    record <- strsplit(lines[3], ',')[[1]] %>% trimws
+    
+    if (length(record) < 2) {
+      warning(sprintf('File %s is incomplete!', fname))
+      res <- NULL
+      info <- NULL
+    } else {
+      res <- matrix(unlist(strsplit(record[-c(1)], ':')), nrow = 2)
+      info <- matrix(as.numeric(unlist(strsplit(res[2, ], '\\|'))), nrow = 2)
+    }
+    
+    record[1] <-  gsub("\\\\", "/", record[1])
+    if ('folder' %in% names(header))
+      datafile <- file.path(path, header$folder, record[1])
+    else
+      datafile <- file.path(path, record[1])
+    
+    
+    # TODO: check the name of the attributes and fix them!
+    data[[i]] <- c(
+      header,
+      list(
+        comment = comment,
+        datafile = datafile,
+        instance = as.numeric(res[1, ]),
+        maxRT = info[1, ],
+        finalFV = info[2, ]
+      )
+    )
+    i <- i + 1
+  }
+  close(f)
+  data
+}
+
 #' Read bi-objective COCO-based .info files and extract information
 #'
 #' @param fname The path to the .info file
@@ -178,7 +256,14 @@ read_index_file__BIOBJ_COCO <- function(fname) {
     else
       datafile <- file.path(path, record[3])
 
-    funcId <- as.numeric(trimws(strsplit(record[1], '=')[[1]][2]))
+    funcId <- trimws(strsplit(record[1], '=')[[1]][2])
+    funcId.int <- suppressWarnings(as.integer(funcId.int))
+    if(!any(is.na(funcId.int))) {
+      if(all((funcId.int >= 0L) & (funcId.int <= 1000000000L))) {
+        funcId <- funcId.int
+      }
+    }
+
     DIM <- as.numeric(trimws(strsplit(record[2], '=')[[1]][2]))
 
     # TODO: check the name of the attributes and fix them!
@@ -205,31 +290,41 @@ read_index_file__BIOBJ_COCO <- function(fname) {
 #' Throws a warning when multiple formats are found in the same folder.
 #'
 #' @param path The path to the folder to check
-#' @return The format of the data in the given folder. Either 'COCO' or 'IOHprofiler'.
+#' @return The format of the data in the given folder. Either 'COCO', 'IOHprofiler',
+#' 'NEVERGRAD' or 'SOS'.
 #' @export
-#' @examples 
+#' @examples
 #' path <- system.file("extdata", "ONE_PLUS_LAMDA_EA", package = "IOHanalyzer")
 #' check_format(path)
 check_format <- function(path) {
   if (sub('[^\\.]*\\.', '', basename(path), perl = T) == "csv")
     return(NEVERGRAD)
-  
+
   index_files <- scan_index_file(path)
+  if (length(index_files) == 0) 
+    return(SOS)
+  
   info <- unlist(lapply(index_files, read_index_file), recursive = F)
   datafile <- sapply(info, function(item) item$datafile)
-  
+
   format <- lapply(datafile, function(file) {
     tryCatch({
+      if (!file.exists(file)) {
+        cdatfile <- stri_replace(file, ".cdat", fixed = ".dat")
+        tdatfile <- stri_replace(file, ".tdat", fixed = ".dat")
+        if (file.exists(cdatfile)) file <- cdatfile
+        else file <- tdatfile
+      }
       first_line <- scan(file, what = 'character', sep = '\n', n = 1, quiet = T)
     }, error = function(e) {
-      stop("Error detecting data files specified in .info, please verify the 
+      stop("Error detecting data files specified in .info, please verify the
              integrity of the provided files.")
     })
-    
+
     if (startsWith(first_line, '% function') || startsWith(first_line, '% f evaluations'))
       COCO
     else if (startsWith(first_line, '\"function')) {
-      n_col <- ncol(fread(file, header = FALSE, sep = ' ', 
+      n_col <- ncol(fread(file, header = FALSE, sep = ' ',
                          colClasses = 'character', fill = T, nrows = 1))
       if (n_col == 2)
         TWO_COL
@@ -245,11 +340,15 @@ check_format <- function(path) {
   }) %>%
     unlist %>%
     unique
-  
+
   csv_files <- file.path(path, list.files(path, pattern = '.csv', recursive = T))
   if (length(csv_files) > 0)
     format <- c(format, NEVERGRAD)
   
+  txt_files <- file.path(path, list.files(path, pattern = '.txt', recursive = T))
+  if (length(txt_files) > 0)
+    format <- c(format, SOS)
+
   if (length(format) > 1) {
     stop(
       paste(
@@ -329,7 +428,7 @@ read_dat__COCO <- function(fname, subsampling = FALSE) {
   df <- fread(text = X[-idx], header = F, sep = ' ', select = select, fill = T)
   idx <- c((idx + 1) - seq_along(idx), nrow(df))
 
-  lapply(seq(length(idx) - 1), 
+  lapply(seq(length(idx) - 1),
          function(i) {
            i1 <- idx[i]
            i2 <- idx[i + 1] - 1
@@ -359,89 +458,6 @@ read_dat__BIOBJ_COCO <- function(fname, subsampling = FALSE) {
   })
 }
 
-# #' Read Nevergrad data file
-# #' 
-# #' Read .csv files in nevergrad format and extract information as a DataSetList
-# #'
-# #' @param fname The path to the .csv file
-# #' @return The DataSetList extracted from the .csv file provided
-# #' @noRd
-# read_nevergrad <- function(path) {
-#   dt <- fread(path)
-  
-#   triplets <- unique(dt[, .(optimizer_name, dimension, name)])
-#   algIds <- unique(triplets$optimizer_name)
-#   DIMs <- unique(triplets$dimension)
-#   funcIds <- unique(triplets$name)
-  
-#   res <- list()
-#   idx <- 1
-  
-#   for (i in seq(nrow(triplets))) {
-#     algId <- triplets$optimizer_name[i]
-#     DIM <- triplets$dimension[i]
-#     funcId <- triplets$name[i]
-    
-#     if (!('rescale' %in% colnames(dt))) {
-#       if ('transform' %in% colnames(dt))
-#         colnames(dt)[colnames(dt) == 'transform'] <- 'rescale'
-#       else
-#         dt$rescale <- NA
-#     }
-    
-#     data <- dt[optimizer_name == algId & 
-#                  dimension == DIM & 
-#                  name == funcId,
-#                .(budget, loss, rescale)]
-    
-#     for (scaled in unique(data$rescale)) {
-#       if (!is.na(scaled)) {
-#         data_reduced <- data[rescale == scaled, .(budget, loss)]
-#       }
-#       else {
-#         data_reduced <- data[is.na(rescale), .(budget, loss)]
-#       }
-      
-#       if (!is.na(scaled) && scaled) {
-#         funcId_name <- paste0(funcId, '_rescaled')
-#       }
-#       else {
-#         funcId_name <- funcId
-#       }
-      
-#       rows <- unique(data_reduced$budget) %>% sort
-#       FV <- lapply(rows,
-#                    function(b) {
-#                      data_reduced[budget == b, loss]
-#                    }
-#       ) %>%
-#         do.call(rbind, .) %>%
-#         set_rownames(rows)
-      
-#       RT <- list()
-#       ds <- structure(
-#         list(RT = RT, FV = FV),
-#         class = c('DataSet', 'list'),
-#         maxRT = max(rows),
-#         finalFV = min(FV),
-#         format = 'NEVERGRAD',
-#         maximization = FALSE,
-#         algId = algId,
-#         funcId = funcId_name,
-#         DIM = DIM
-#       )
-#       res[[idx]] <- ds
-#       idx <- idx + 1
-#     }
-#   }
-  
-#   class(res) <- c(class(res), 'DataSetList')
-#   attr(res, 'DIM') <- DIMs
-#   attr(res, 'funcId') <- funcIds
-#   attr(res, 'algId') <- algIds
-#   res
-# }
-
 # global variables for the alignment functions
 idxEvals <- 1
 idxTarget <- 3
@@ -502,7 +518,7 @@ align_running_time <- function(data, format = IOHprofiler, include_param = TRUE,
   else if (format == BIBOJ_COCO) {
     n_data_column <- 3
     idxTarget <- 2
-  } 
+  }
   else if (format == TWO_COL) {
     n_data_column <- 2
     idxTarget <- 2
@@ -516,23 +532,23 @@ align_running_time <- function(data, format = IOHprofiler, include_param = TRUE,
     n_param <- 0
     idxValue <- idxEvals
     param_names <- NULL
-  } 
+  }
   else if (format == IOHprofiler) {
     n_param <- n_column - n_data_column
     if (include_param && n_param > 0) {
       param_names <- colnames(data[[1]])[(n_data_column + 1):n_column]
       idxValue <- c(idxEvals, (n_data_column + 1):n_column)
-    } 
+    }
     else {
       param_names <- NULL
       idxValue <- idxEvals
     }
-  } 
+  }
   else {
     param_names <- NULL
     idxValue <- idxEvals
   }
-  
+
   res <- c_align_running_time(data, FV, idxValue - 1, maximization, idxTarget - 1)
   names(res) <- c('RT', param_names)
   res
@@ -552,18 +568,18 @@ align_function_value <- function(data, include_param = TRUE, format = IOHprofile
     maximization <- FALSE
     idxTarget <- 3
     n_param <- 0
-  } 
+  }
   else if (format == IOHprofiler) {
     maximization <- TRUE
     idxTarget <- 3
     n_param <- n_column - n_data_column
-  } 
+  }
   else if (format == BIBOJ_COCO) {  # bi-objective COCO format
     maximization <- FALSE
     idxTarget <- 2
     n_data_column <- 2
     n_param <- 0                   # no parameter is allowed in this case
-  } 
+  }
   else if (format == TWO_COL) {
     maximization <- TRUE
     idxTarget <- 2
@@ -582,7 +598,7 @@ align_function_value <- function(data, include_param = TRUE, format = IOHprofile
 
   FV <- align_func(data, idxTarget, runtime)
   include_param <- include_param && (n_param > 0)
-  
+
   if (include_param) {
     param_names <- colnames(data[[1]])[(n_data_column + 1):n_column]
     param <- list()
@@ -601,7 +617,7 @@ align_function_value <- function(data, include_param = TRUE, format = IOHprofile
 
 
 #' Read Nevergrad data
-#' 
+#'
 #' Read .csv files in nevergrad format and extract information as a DataSetList
 #'
 #' @param fname The path to the .csv file
@@ -632,7 +648,7 @@ read_nevergrad <- function(path){
         dt$rescale <- NA
       }
     }
-    
+
     data <- dt[optimizer_name == algId & dimension == DIM & name == funcId,
                .(budget, loss, rescale)]
 
@@ -682,5 +698,176 @@ read_nevergrad <- function(path){
   attr(res, 'suite') <- 'NEVERGRAD'
   attr(res, 'maximization') <- F
   res
+
+}
+
+#' Read single DataSet of SOS-based data
+#' 
+#' Read single .txt files in SOS format and extract information as a DataSet
+#'
+#' @param file The path to the .txt file
+#' @return The DataSet extracted from the .txt file provided
+#' @noRd
+read_single_file_SOS <- function(file) {
+  V1 <- NULL #Local binding to remove CRAN warnings
   
+  algId <- substr(basename(file), 1,  stringi::stri_locate_last(basename(file), fixed = 'D')[[1]] - 1)
+  
+  dt <- fread(file, header = F)
+  header <- scan(file, what = 'character', sep = '\n', n = 1, quiet = T)
+  splitted <- header %>% trimws %>% strsplit("\\s+") %>% .[[1]] %>% .[2:length(.)]
+  info <- list(algId = algId)
+  for (i in seq_len(length(splitted) / 2)) {
+    temp <- splitted[[2*i]]
+    name <- splitted[[2*i - 1]]
+    if (name == 'function') name <- 'funcId'
+    if (name == 'dim') name <- 'DIM'
+    names(temp) <- name
+    info <- c(info, temp)
+  }
+  
+  dim <- as.numeric(info$DIM)
+  
+  
+  RT_raw <- dt[[colnames(dt)[[ncol(dt) - dim - 1]]]]
+  names(RT_raw) <- dt[[colnames(dt)[[ncol(dt) - dim - 2]]]]
+  RT <- as.matrix(RT_raw)
+  mode(RT) <- 'integer'
+  
+  FV_raw <- dt[[colnames(dt)[[ncol(dt) - dim - 2]]]]
+  names(FV_raw) <- dt[[colnames(dt)[[ncol(dt) - dim - 1]]]]
+  FV <- as.matrix(FV_raw)
+  
+  
+  pos <- dt[, (ncol(dt) - dim + 1):ncol(dt)]
+  colnames(pos) <- as.character(seq_len(dim))
+  
+  maxRT <- max(RT)
+  finalFV <- min(FV)
+  
+  if (sum(FV == finalFV) > 1) {
+    #Reconstruct population to determine which best solution is final position
+    ids_min <- dt[FV_raw == finalFV, V1]
+    replaced_idxs <- dt[[colnames(dt)[[ncol(dt) - dim]]]]
+    #If none, take the last one added
+    pos_idx <- max(ids_min)
+    for (i in ids_min) {
+      if (all(replaced_idxs != i)) {
+        #If multiple, take the first one added
+        pos_idx <- i
+        break
+      }
+    }
+    final_pos <- as.numeric(pos[pos_idx, ])
+  }
+  else {
+    final_pos <- as.numeric(pos[which.min(FV), ])
+  }
+  
+  PAR <- list(
+    # 'position' = list(pos),
+    'final_position' = list(final_pos),
+    'by_FV' = NULL,
+    'by_RT' = NULL
+  )
+  
+  
+  
+  object <- list()
+  class(object) <- c('DataSet', class(object))
+  object$RT <- RT
+  object$FV <- FV
+  object$PAR <- PAR
+  attr(object, 'maxRT') <- maxRT
+  attr(object, 'finalFV') <- finalFV
+  attr(object, 'format') <- "SOS"
+  attr(object, 'maximization') <- F
+  attr(object, 'suite') <- "SOS"
+  for (i in seq_along(info)) {
+    attr(object, names(info)[[i]]) <- type.convert(info[[i]], as.is = T)
+  }
+  object
+}
+
+
+#' Read DataSetList of SOS-based data
+#' 
+#' Read directory containing .txt files in SOS format and extract information as a DataSetList
+#'
+#' @param dir The path to the directory file
+#' @param corrections_file A file containing boundary-correction ratios for the files in `dir`
+#' @return The DataSetList extracted from the directory provided
+#' @noRd
+read_datasetlist_SOS <- function(dir, corrections_files = NULL) {
+  V1 <- V3 <- V4 <- NULL #Local binding to remove CRAN warnings
+  res <- list()
+  dims <- list()
+  funcIds <- list()
+  algIds <- list()
+  suites <- list()
+  maximizations <- list()
+  
+  idx <- 1
+  
+  corrs <- as.data.table(rbindlist(lapply(corrections_files, fread)))
+  
+  for (f in list.files(dir, recursive = T, pattern = "*.txt", full.names = T)) {
+    if (f %in% corrections_files) next
+    ds <- read_single_file_SOS(f)
+    
+    dims[[idx]] <- attr(ds, 'DIM')
+    funcIds[[idx]] <- attr(ds, 'funcId')
+    algIds[[idx]] <- attr(ds, 'algId')
+    suites[[idx]] <- attr(ds, 'suite')
+    maximizations[[idx]] <- attr(ds, 'maximization')
+    
+    if (nrow(corrs) > 0) {
+      fn <- substr(basename(f), 1, nchar(basename(f)) - 4)
+      corr_opts <- corrs[V1 == fn, ]
+      if (stri_detect_fixed(fn, "DE")) {
+        corr <- corr_opts[V3 == attr(ds, 'F'), ][V4 == attr(ds, 'CR'), 'V2'][['V2']]
+      }
+      else if (stri_detect_fixed(fn, "RIS")) {
+        corr <- corr_opts[['V2']]
+      }
+      else {
+        warning("Unknown algorithm, so skipping lookup of boundary corrections ratio")
+        corr <- NULL
+      }
+      if (length(corr) == 1)
+        ds$PAR$'corrections' <- corr[[1]]
+      else
+        warning(paste0("No boundary corrections ratio found for ", fn))
+    }
+    
+    res[[idx]] <- ds
+    idx <- idx + 1
+  }
+  class(res) %<>% c('DataSetList')
+  attr(res, 'DIM') <- dims
+  attr(res, 'funcId') <- funcIds
+  attr(res, 'algId') <- algIds
+  
+  suite <- unique(suites)
+  maximization <- unique(maximizations)
+  if (length(suite) != 1 || length(maximization) != 1) {
+    warning("Multipe different suites detected!")
+  }
+  
+  attr(res, 'suite') <- suite
+  attr(res, 'maximization') <- maximization
+  res
+  clean_DataSetList(res)
+}
+
+#' Find corrections-files in SOS-based folder
+#' 
+#' Read directory containing .txt files in SOS format and extract the corrections-files
+#'
+#' @param path The path to the directory file
+#' @return The relative paths to the corection files
+#' @noRd
+locate_corrections_files <- function(path) {
+  files <- list.files(path, recursive = T, pattern = "*.txt", full.names = T)
+  files[stri_detect_fixed(files, 'corrections')]
 }
